--
-- Licensed Materials - Property of IBM Corp. Copyright (c) 1997, 2010
-- 5724-X63, 5724-W86
-- Copyright IBM Corporation 2007. All Rights Reserved.
-- US Government Users Restricted Rights- Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
--
Set Define "&"
Set Verify Off
Set ServerOutput On Size 1000000
Prompt = = = = = = REMOVE SPARSE WORKING TABLE SYNS FOR CHANNEL 99 = = = = = =
Begin
  For recSyn In ( Select Owner
                    From DBA_SYNONYMS
                   Where Synonym_Name = 'C99_SNRW_000_S0' )
  Loop
    Execute Immediate 'Drop Synonym '||recSyn.Owner||'.C99_SNRW_000_S0';
  End Loop;
End; 
/
Prompt = = = = = = = REMOVE SPARSE WORKING TABLES FOR CHANNEL 99 = = = = = = =
Declare
  vcMetric         Varchar2(40) := Pvm_Client.Get_Root||'_METRIC';
Begin
  For recTab In ( Select 'X'
                    From DBA_TABLES
                   Where Table_Name = 'C99_SNRW_000_S0'
                     And Owner = vcMetric )
  Loop
    Pvm_Table.Drop_Table('C99_SNRW_000_S0', True);
  End Loop;
End; 
/
Prompt = = = = = = REMOVE SPARSE WORKING TABLESPACE FOR CHANNEL 99 = = = = = =
Exec Pvm_Tablespace.Drop_Tablespace('C99_SNRW_000_0000000000');
Prompt = = = = = = RECREATE SPARSE WORKING TABLES TO METRIC SCHEMA = = = = = =
Declare
  ERR_CANT_FORMAT_TAB_NAME  Constant PLS_Integer := -20200;
  ERR_INVALID_TBSPACE_NAME  Constant PLS_Integer := -20150;
  vcRegBaseKey              Varchar2(1024);
  vcSetType                 Varchar2(5) := 'SNRW';
  vcCurrentTablespace       Varchar2(32);
  iStatus                   PLS_Integer;
  vcTableName               Varchar2(32);
  iTableCat                 PLS_integer;
  adRegKey                  Pvm_Registry.Address;
  iPartSizeKB               Integer;
  iPartExtents              Integer;
  iExtentSizeKB             Integer;
  Function Local_Table_Exists( P_vcName In Varchar2 ) Return Boolean
  Is
    Cursor curTable Is
          Select Null
            From Dba_Tables
           Where Upper(Table_Name) = Upper(P_vcName);
    cDummy      Char(1);
    bResult     Boolean;
  Begin
    Open curTable;
    Fetch curTable Into cDummy;
    bResult := curTable%Found;
    Close curTable;
    Return( bResult );
  Exception
     When OTHERS Then
        If curTable%ISOPEN then
          Close curTable;
        End if;
        Pvm_Error.Raise_Error( SQLCODE );
        Raise;
  End Local_Table_Exists;
Begin
  For recChnl In ( Select Idx_Ind
                     From Chnl_Desc
                    Where Int_Status = 1 )
  Loop
    vcSetType    := 'SNRW';
    vcRegBaseKey := Pvm_Chnl_Mgr.Get_Base_Key( recChnl.Idx_Ind );
    iStatus := Pvm_Name.Format_Channel_TS_Name( vcCurrentTablespace,
                                                recChnl.Idx_Ind,
                                                vcSetType,
                                                0,
                                                Pvm_Name.TSTAMP_WORKING );
    If iStatus != 0 Then
       Pvm_Error.Raise_Error( ERR_INVALID_TBSPACE_NAME );
    End If;
    iStatus := Pvm_Name.Format_Channel_Table_Name( vcTableName,
                                                   recChnl.Idx_Ind,
                                                   vcSetType,
                                                   0,
                                                   'S0' );
    If iStatus != 0 Then
       Pvm_Error.Raise_Error( ERR_CANT_FORMAT_TAB_NAME );
    End If;
    If Local_Table_Exists( vcTableName ) Then
      PVM_Table.Drop_Table( vcTableName, True );
    End If;
    iTableCat := Pvm_Table.Determine_Table_Category( vcSetType );
    adRegKey := Pvm_Registry.Get_Address( vcRegBaseKey ||
                                          vcSetType ||
                                          '.TABLE.CURRENT' );
    iPartSizeKB := Pvm_Utility.To_KBytes(
                     Pvm_Registry.Get_Size_Value( adRegKey,
                                                  'PARTITION_SIZE' ),
                     'UP' );
    iPartExtents := Greatest( Pvm_Registry.Get_Number_Value(
                                                  adRegKey,
                                                  'PARTITION_EXTENTS' ),
                                1 );
    iExtentSizeKB := Trunc( iPartSizeKB / iPartExtents );
    If Mod( iPartSizeKB, iPartExtents ) != 0 Then
      iExtentSizeKB := iExtentSizeKB + 1;
    End If;
    PVM_Table.Create_Table( iTableCat,
                            vcTableName,
                            iExtentSizeKB * 1024,
                            iExtentSizeKB * 1024,
                            vcCurrentTablespace );
    Dbms_Stats.Gather_Table_Stats( ownname => Pvm_Name.Get_Metric_Schema,
                                   tabname => vcTableName );
  End Loop;
Exception
  When OTHERS Then
    Pvm_Error.Raise_Error( SQLCODE );
    Raise;
End;
/
Prompt = = = = = = = SET SPARSE HISTORY TABLESPACES TO AUTOEXTEND  = = = = = =
Begin
  For recTbsp In ( Select Tablespace_Name
                     From DBA_TABLESPACES
                    Where Tablespace_Name Like 'C99\_SNRW\_000'||
                                               '\_000000000_' Escape '\' )
  Loop
    Pvm_Tablespace.Set_Last_Chnl_Dbf_Autoextend(recTbsp.Tablespace_Name, 50, 0);
  End Loop;
End;
/
Prompt = = = = = = = = =  MIGRATE SPARSE HISTORY SUPPORT = = = = = = = = = = =
Declare
  iCurrentDate     Number := Pvm_Time.Date2Second(Pvm_Time.Get_GMT_Date);
  vcMetric         Varchar2(40) := Pvm_Client.Get_Root||'_METRIC';
  vcSNRW           Varchar2(32) := 'C99_SNRW_000_H1';
  vcSNRWTmp        Varchar2(32) := 'C99_SNRW_000_H1_TMP';
  vcRegBaseKey     Varchar2(255) := Pvm_Chnl_Mgr.Get_Base_Key(99);
  adRegKey         Pvm_Registry.Address;
  vcHistTablespace Varchar2(32) := 'C99_SNRW_000_';
  iTableCat        PLS_integer;
  iTabCnt          Number;
  iPartCnt         Number;
  iPartSizeKB      Integer;
  iPartExtents     Integer;
  iExtentSizeKB    Integer;
  iOldRowCnt       Number;
  iNewRowCnt       Number;
Begin
  If Not Temp_SNRW_Migrated Then
    Select Count(*)
      Into iTabCnt
      From DBA_TABLES
     Where Table_Name = vcSNRWTmp;
    If iTabCnt = 0 Then 
      Temp_Execute('Alter Table '||vcSNRW||' Rename To '||vcSNRWTmp);
      Temp_Execute('Alter Index PK_'||vcSNRW||' Rename To PK_'||vcSNRWTmp);
      Temp_Execute('Alter Table '||vcSNRWTmp||' Rename Constraint PK_'||vcSNRW||' To PK_'||vcSNRWTmp);
    End If;
    For recPart In ( Select Partition_Name
                       From DBA_TAB_PARTITIONS
                      Where Table_Name = vcSNRWTmp
                        And Table_Owner = vcMetric )
    Loop
      If recPart.Partition_Name Not Like '%_TMP' Then
        Execute Immediate('Alter Table '||vcMetric||'.'||vcSNRWTmp||' Rename Partition '||
                          recPart.Partition_Name||' To '||recPart.Partition_Name||'_TMP');
      End If;
    End Loop;
    iTableCat := Pvm_Table.Determine_Table_Category('SNRW');
    adRegKey := Pvm_Registry.Get_Address( vcRegBaseKey||'SNRW.TABLE.HISTORIC' );
    iPartSizeKB := Pvm_Utility.To_KBytes(
                     Pvm_Registry.Get_Size_Value( adRegKey,
                                                  'PARTITION_SIZE' ),
                     'UP' );
    iPartExtents := Greatest( Pvm_Registry.Get_Number_Value(
                                                  adRegKey,
                                                  'PARTITION_EXTENTS' ),
                              1 );
    iExtentSizeKB := Trunc( iPartSizeKB / iPartExtents );
    If Mod( iPartSizeKB, iPartExtents ) != 0 Then
      iExtentSizeKB := iExtentSizeKB + 1;
    End If;
    Select Count(*)
      Into iTabCnt
      From DBA_TABLES
     Where Table_Name = vcSNRW;
    If iTabCnt = 0 Then 
      PVM_Table.Create_Table( iTableCat,
                              vcSNRW,
                              iExtentSizeKB * 1024,
                              iExtentSizeKB * 1024,
                              vcHistTablespace,
                              Pvm_Name.TSTAMP_SPARSE );
      Dbms_Stats.Gather_Table_Stats( ownname     => vcMetric,
                                     tabname     => vcSNRW,
                                     method_opt  => 'FOR ALL COLUMNS SIZE 1' );
    Else
      Execute Immediate 'Truncate Table '||vcMetric||'.'||vcSNRW;
    End If;
    Execute Immediate 'Insert Into '||vcMetric||'.'||vcSNRW||
                      ' (Idx_Metric, Idx_Resource, Dte_Date, Dte_Adj_Date, '||
                      '  Str_Action, Dbl_Value, Idx_Event ) '||
                      ' Select Idx_Metric, Idx_Resource, Dte_Date, Dte_Adj_Date, '||
                      '        ''I'', Dbl_Value, Idx_Event '||
                      '  From '||vcMetric||'.'||vcSNRWTmp;
    Execute Immediate 'Select Count(*) From '||vcMetric||'.'||vcSNRW Into iNewRowCnt;
    Execute Immediate 'Select Count(*) From '||vcMetric||'.'||vcSNRWTmp Into iOldRowCnt;
    If iNewRowCnt = iOldRowCnt Then
      Pvm_Config.Set_Number_Value('DATABASE.GENERAL.SNRW.SNRW_MIGRATE_DATE',iCurrentDate,True,True);
      Execute Immediate 'Drop Table '||vcMetric||'.'||vcSNRWTmp;
    Else
      Pvm_Error.Raise_Error( -20600, 'Unexpected failure during SNRW migration' );
    End If; 
  End If;
Exception
  When OTHERS Then
    Raise;
End;
/
Prompt = = = = = = = = = = = UPDATING VIEW DEFINITIONS = = = = = = = = = = =
Declare
  vcSetType     Varchar2(5) := 'SNRW';
Begin
  For recView In ( Select Distinct
                          t.Str_Name VwType, a.Idx_Ind AggNum
                     From Chnl_Table_Type t, Chnl_Table_Set_Type s, AggSet_Desc a
                    Where t.Str_Name Like 'H_'
                      And a.Int_Status > 0
                      And s.Str_Name = vcSetType
                      And NVL( 0, a.Idx_Ind ) = a.Idx_Ind
                      And ( s.Int_Aggset_Enabled = 1 Or a.Idx_Ind = 0 )
                    Union All
                   Select Distinct
                          v.Str_Name VwType, a.Idx_Ind AggNum
                     From Chnl_View_Type v, Chnl_Table_Set_Type s, AggSet_Desc a
                    Where a.Int_Status > 0
                      And s.Str_Name = vcSetType
                      And NVL( 0, a.Idx_Ind ) = a.Idx_Ind
                      And ( s.Int_Aggset_Enabled = 1 Or a.Idx_Ind = 0 ) )
  Loop
    Dbms_Output.Put_Line('Creating sparse views for '||recView.VwType);
    For recC In ( Select Distinct Substr( Table_Name, 2, 2 ) As c_ChnlNum
                    From All_Tables
                   Where Owner = Pvm_Name.Get_Metric_Schema )
    Loop
      If recC.c_ChnlNum != Pvm_Chnl_Mgr.Get_Sparse_Chnl_Number Then
        Pvm_Chnl_View.Create_Timestamped_View( recC.c_ChnlNum,
                                               vcSetType,
                                               recView.AggNum,
                                               recView.VwType,
                                               0 );
      End If;
    End Loop;
  End Loop;
  Delete
    From CHNL_VIEW_DESC
   Where Idx_Table_Set_Type = 10;
  Commit;
End;
/
Prompt = = = = = = = UPDATING CROSS CHANNEL VIEW DEFINITIONS  = = = = = = = =
Exec Pvm_Chnl_View.Create_Replace_C00_View;
Prompt = = = = = = = = = UPDATING GRANTS FOR SPARSE TABLES  = = = = = = = = =
Begin
  For recC In ( Select Distinct Substr( Table_Name, 2, 2 ) As c_ChnlNum
                  From All_Tables Where Owner = Pvm_Name.Get_Metric_Schema )
  Loop
    Dbms_Output.Put_Line('Creating synonyms and grants for channel '||recC.c_ChnlNum);
    Pvm_Chnl_Mgr.Create_Syn_And_Grant( recC.c_ChnlNum, 0 );
  End Loop;
End;
/
